#include "config.h"
#include "adc.h"
#include "dac.h"
#include "uart.h"
#include "timer.h"
#include <stdio.h>
#include <math.h>
#include "scaling.h"
#include "util.h"
#include "led.h"
#include "mode.h"
#include "zoom.h"

#define DEBUG_BLINK_RATE 4

/* Debug LED */
void TISR_HANDLER(6)
{	
	static int toggle = 0;
	timer_clear_txif(6);
	
	toggle = !toggle;
	if (toggle)
		led_on();
	else
		led_off();
}

static uint16_t dac = 32768;

#define TIMER_RATE 8000
static int zero_adc = 0;

void TISR_HANDLER(7)
{
	int16_t v;
	float dac_current, i;

	timer_clear_txif(7);

	if (zero_adc) {
		dac_current = dac_to_current(dac);

		v = adc_get();
		i = adc12_to_current(v);
		/* adjust by 50% of the estimated value, in case our calibration is wrong */
		dac_current -= (i * 0.5);
		if (dac_current < dac_current_min)
			dac_current = dac_current_min;
		if (dac_current > dac_current_max)
			dac_current = dac_current_max;
		dac = current_to_dac(dac_current);
		dac_write(dac);
	}
}

void sweep(void)
{
	int32_t d;
	int16_t a;	

	/* sweep range */
	for (d = (int32_t)dac - 1500; d < (int32_t)dac + 1500; d++) {
		if (d < 0x0000) {
			uart1_put_string("0 0\r\n");
			continue;
		}
		if (d > 0xffff) {
			uart1_put_string("65535 0\r\n");
			continue;
		}
		dac_write(d);
		msleep(1);
		a = adc_get();
		uart1_put_dec(d);
		uart1_put(' ');
		uart1_put_dec(a);
		uart1_crlf();
	}
}

void run_debug(void)
{
	int16_t adc;
	int32_t v;
	char buf[4];
	uart1_init(115200);	

	timer_setup_16bit(6, DEBUG_BLINK_RATE, 1);
	timer_setup_16bit(7, TIMER_RATE, 1);

	uart1_put_string("Zoom NILM Debug\r\n");

	while (1) {
		dac_write(dac);
		uart1_put_hex16(dac);
		uart1_put(' ');
		uart1_put_dec(dac);
		uart1_put(' ');
		uart1_put(' ');
		adc = adc_get();
		uart1_put_hex16(adc);
		uart1_put(' ');
		uart1_put_dec(adc);
		uart1_crlf();
		switch (uart1_get()) {

		// small step
		case '[':
			dac--;
			break;
		case ']':
			dac++;
			break;

		// medium step
		case '-':
			dac -= 16;
			break;
		case '+':
		case '=':
			dac += 16;
			break;

		// big step
		case ',':
		case '<':
			dac -= 1024;
			break;
		case '.':
		case '>':
			dac += 1024;
			break;

		// set DAC to midpoint
		case '0':
			dac = 32768;
			break;

		// run degauss (currently a noop)
		case 'd':
			uart1_put_string("degauss...");
			degauss();
			uart1_crlf();
			dac = 32768;
			break;

		// set DAC to specified hex value
		case 'v':
		case 'V':
			buf[0] = uart1_get();
			buf[1] = uart1_get();
			buf[2] = uart1_get();
			buf[3] = uart1_get();
			v = hex_to_u16(buf);
			if (v < 0)
				uart1_put_string("bad value\r\n");
			else
				dac = v;
			break;

		// maintain ADC input at zero
		case 'z':
		case 'Z':
			uart1_put_string("zeroing input...\r\n");
			zero_adc = 1;
			uart1_get();
			zero_adc = 0;
			break;

		// sweep DAC
		case 's':
		case 'S':
			uart1_put_string("sweep around ");
			uart1_put_dec(dac);
			uart1_crlf();
			sweep();
			break;
		}
	}
}

